/*
 * File:	DemoUART.cpp
 *
 * Date:	2012-11-28
 * Company:	DH electronics GmbH
 * Author:	Ludwig Zenz, lzenz@dh-electronics.de
 *
 * Remark:	The following code aims to offer you some help
 * 			with the UART operations in Linux user-space programs.
 */


/*
 * For further  documentation and examples have a look at:
 * ->	http://www.kernel.org/doc/Documentation/serial/
 * -> 	http://linux.die.net/man/3/tcgetattr
 * ->   http://www.easysw.com/~mike/serial/serial.html#2_3_2
 */


/* Header */
#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>



#define DHCOM_AM35x
#ifdef DHCOM_AM35x
#define DHCOM_FFUART 	"/dev/ttyO0"
#define DHCOM_BTUART 	"/dev/ttyO2"
#define DHCOM_STDUART 	"/dev/ttyO1"
#else /* DHCOM_iMX25 */
#define DHCOM_FFUART 	"/dev/ttymxc1"
#define DHCOM_BTUART 	"/dev/ttymxc2"
#define DHCOM_STDUART 	"/dev/ttymxc4"
#endif


int main()
{
	int ret, file, status, bytes, length;
	struct termios argp;
	void *buffer;
	fd_set input_fdset;

	printf("\nStarting UART demo ...\n\n");

	/*************************************************************************************
	 * The first thing to do is to open a device-file
	 */

	file = open(DHCOM_FFUART, O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (file == -1)
	{
	    printf("ERROR: Can't open device: ");
	    printf(DHCOM_FFUART);
	    return (-1);
	}

	/*************************************************************************************
	 * configure connection
	 */
	ret = ioctl(file, TCGETS, &argp);
		if (ret < 0) { printf("ERROR during ioctl TCGETS! \n"); }

	/* No parity (8N1) */
	argp.c_cflag &= ~PARENB;
	argp.c_cflag &= ~CSTOPB;
	argp.c_cflag &= ~CSIZE;
	argp.c_cflag |= CS8;

	/* set baudrate to 115200 kbit/s */
	argp.c_cflag &= ~(CBAUD);
	argp.c_cflag |= B38400;

	/* enable hw flowcontrol RTS/CTS*/
	argp.c_cflag |= CRTSCTS;

	/* using raw input: input characters are passed through exactly as they are received */
	argp.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

	ret = ioctl(file, TCSETS, &argp);
		if (ret < 0) { printf("ERROR during ioctl TCSETS! \n"); }


	/*************************************************************************************
	 * getting modem status
	 */

	ret = ioctl(file, TIOCMGET, &status);
		if (ret < 0) { printf("ERROR during ioctl TIOCMGET! \n"); }

	 if (status & TIOCM_LE)
		 printf("DSR (data set ready / line enable) is set \n");

	 if (status & TIOCM_DTR)
		 printf("DTR (data terminal ready) is set \n");

	 if (status & TIOCM_RTS)
		 printf("RTS (request to send) is set \n");

	 if (status & TIOCM_CTS)
		 printf("CTS (clear to send) is set \n");

	 if (status & TIOCM_CD)
		 printf("DCD (data carrier detect) is set \n");

	 if (status & TIOCM_RI)
		 printf("RNG (ring) is set \n");

	 if (status & TIOCM_DSR)
		 printf("DSR (data set ready) is set \n");

	/*************************************************************************************
	 * setting modem status
	 */

	ret = ioctl(file, TIOCMGET, &status);
		if (ret < 0) { printf("ERROR during ioctl TIOCMGET! \n"); }

	    status &= ~TIOCM_DTR; /* Dropping DTR */

	ret = ioctl(file, TIOCMSET, &status);
		if (ret < 0) { printf("ERROR during ioctl TIOCMSET! \n"); }

	/*************************************************************************************
	 * write some test data
	 */

	write(file, "test!", 5);

	/*************************************************************************************
	 * get number of bytes available and read it
	 */

	ret = ioctl(file, FIONREAD, &bytes);
		if (ret < 0) { printf("ERROR during ioctl FIONREAD! \n"); }

	printf ("available bytes in input buffer: %d \n", bytes);

	buffer = malloc(bytes + 1);

	length = read(file, buffer, bytes); /* read available data */

	printf ("Read data: \n");
	printf("%s", (char*)buffer);

	printf ("\n");

	/*************************************************************************************
	 * close the device-file
	 */
	close(file);

	return 0;
}


